//
// ERGOT : The Windows MacTierra clone
// copyright (c) 2005 plushpuffin@gmail.com
// http://www.wwddfd.com/
//
// latest build: http://www.wwddfd.com/frell/ergot.zip
//
// licensing terms still under consideration...

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

namespace ERGOT
{
	/// <summary>
	/// Summary description for Census.
	/// </summary>
	public class CensusForm : System.Windows.Forms.Form
	{
		internal Machine ma = null;
		internal bool pauseUpdates = false;
		internal DateTime lastUpdate = DateTime.MinValue;
		internal System.Collections.Hashtable species = new System.Collections.Hashtable(101);
		private bool _isDisposed = false;
		//private censusListSorter sorter = null;

		private System.Windows.Forms.ListView lvGeneBank;
		private System.Windows.Forms.ImageList imgs_FDNormal;
		private System.Windows.Forms.ColumnHeader col_Species;
		private System.Windows.Forms.ColumnHeader col_Population;
		private System.Windows.Forms.ColumnHeader col_TotalPop;
		private System.Windows.Forms.ColumnHeader col_Peak;
		private System.ComponentModel.IContainer components;

		public CensusForm()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			this.DialogResult = DialogResult.None;
		}

		#region register Machine
        public void register( Machine newMa )
		{
			if( this.ma != null )
			{
				ma.MachineRefresh -= new MachineRefreshHandler(this.Machine_Report_Refresh_UI);
				ma.MachineStop -= new MachineRefreshHandler(this.Machine_Stop_Refresh_UI);
			}

			this.ma = null;
			species.Clear();
			lastUpdate = DateTime.MinValue;

			if( newMa == null ) { return; }

			ma = newMa;
			ma.MachineRefresh += new MachineRefreshHandler(this.Machine_Report_Refresh_UI);
			ma.MachineStop += new MachineRefreshHandler(this.Machine_Stop_Refresh_UI);

			updateDisplay();
		}
		#endregion

		#region Refresh/Update
		private void Machine_Report_Refresh_UI(object sender, MachineState m)
		{
			TimeSpan diff = DateTime.Now.Subtract(lastUpdate);
			if( diff.Seconds < 5 ) { return; }
			this.updateDisplay();
		}

		private void Machine_Stop_Refresh_UI(object sender, MachineState m)
		{
			this.updateDisplay();
		}

		public void updateDisplay()
		{
			if( pauseUpdates ) { return; }
			lock( lvGeneBank ) 
			{
				lastUpdate = DateTime.Now;
				lvGeneBank.BeginUpdate();

				geneBankRecord curGBR;
				ListViewItem lvItem;

				for( int i = lvGeneBank.Items.Count - 1; i >= 0; --i )
				{
					lvItem = lvGeneBank.Items[i];
					if( ! ma.myGeneBank.ContainsSpecies(lvItem.Text) )
					{
						lvGeneBank.Items.RemoveAt(i);
						species.Remove(lvItem.Text);
					}
				}

				KTL.Node cur = ma.myGeneBank._genebank.Head;
			
				while( cur != null && ! cur.isNull )
				{
					curGBR = ((geneBankRecord)(cur.Value));
					lvItem = (ListViewItem)(species[curGBR.mySpeciesName]);
					if( lvItem == null )
					{
						lvItem = new ListViewItem();
						lvItem.ImageIndex = 2;
						lvItem.Tag = curGBR;
						lvGeneBank.Items.Add(lvItem);
						lvItem.Text = curGBR.mySpeciesName;
						species.Add(curGBR.mySpeciesName,lvItem);
						lvItem.SubItems.Add(curGBR.myCurrentPopulation.ToString().PadLeft(8,' '));
						lvItem.SubItems.Add(curGBR.myCount.ToString().PadLeft(8,' '));
						lvItem.SubItems.Add(curGBR.myPeak.ToString().PadLeft(8,' '));
					}
					else
					{
						lvItem.SubItems[1].Text = curGBR.myCurrentPopulation.ToString().PadLeft(8,' ');
						lvItem.SubItems[2].Text = curGBR.myCount.ToString().PadLeft(8,' ');
						lvItem.SubItems[3].Text = curGBR.myPeak.ToString().PadLeft(8,' ');
					}
					cur = cur.NextNode;
				}
				lvGeneBank.EndUpdate();
			}
		}

		#endregion

		#region Properties
		public bool isDisposed
		{
			get { return _isDisposed; }
		}
		#endregion

		#region Dispose
		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( ma != null )
			{
				ma.MachineRefresh -= new MachineRefreshHandler(this.Machine_Report_Refresh_UI);
				ma.MachineStop -= new MachineRefreshHandler(this.Machine_Stop_Refresh_UI);
			}
			this.DialogResult = DialogResult.Abort;
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
			_isDisposed = true;
		}

		#endregion

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(CensusForm));
			this.lvGeneBank = new System.Windows.Forms.ListView();
			this.col_Species = new System.Windows.Forms.ColumnHeader();
			this.col_Population = new System.Windows.Forms.ColumnHeader();
			this.col_TotalPop = new System.Windows.Forms.ColumnHeader();
			this.col_Peak = new System.Windows.Forms.ColumnHeader();
			this.imgs_FDNormal = new System.Windows.Forms.ImageList(this.components);
			this.SuspendLayout();
			// 
			// lvGeneBank
			// 
			this.lvGeneBank.Anchor = (((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right);
			this.lvGeneBank.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
																						 this.col_Species,
																						 this.col_Population,
																						 this.col_TotalPop,
																						 this.col_Peak});
			this.lvGeneBank.FullRowSelect = true;
			this.lvGeneBank.GridLines = true;
			this.lvGeneBank.LabelEdit = true;
			this.lvGeneBank.LabelWrap = false;
			this.lvGeneBank.LargeImageList = this.imgs_FDNormal;
			this.lvGeneBank.Location = new System.Drawing.Point(8, 8);
			this.lvGeneBank.MultiSelect = false;
			this.lvGeneBank.Name = "lvGeneBank";
			this.lvGeneBank.Size = new System.Drawing.Size(376, 144);
			this.lvGeneBank.SmallImageList = this.imgs_FDNormal;
			this.lvGeneBank.TabIndex = 0;
			this.lvGeneBank.View = System.Windows.Forms.View.Details;
			this.lvGeneBank.AfterLabelEdit += new System.Windows.Forms.LabelEditEventHandler(this.lvGeneBank_AfterLabelEdit);
			this.lvGeneBank.ItemDrag += new System.Windows.Forms.ItemDragEventHandler(this.lvGeneBank_ItemDrag);
			this.lvGeneBank.BeforeLabelEdit += new System.Windows.Forms.LabelEditEventHandler(this.lvGeneBank_BeforeLabelEdit);
			// 
			// col_Species
			// 
			this.col_Species.Text = "Species";
			this.col_Species.Width = 120;
			// 
			// col_Population
			// 
			this.col_Population.Text = "Living Pop.";
			this.col_Population.Width = 80;
			// 
			// col_TotalPop
			// 
			this.col_TotalPop.Text = "Living & Dead";
			this.col_TotalPop.Width = 80;
			// 
			// col_Peak
			// 
			this.col_Peak.Text = "Peak Pop.";
			this.col_Peak.Width = 80;
			// 
			// imgs_FDNormal
			// 
			this.imgs_FDNormal.ColorDepth = System.Windows.Forms.ColorDepth.Depth32Bit;
			this.imgs_FDNormal.ImageSize = new System.Drawing.Size(16, 16);
			this.imgs_FDNormal.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imgs_FDNormal.ImageStream")));
			this.imgs_FDNormal.TransparentColor = System.Drawing.Color.Transparent;
			// 
			// CensusForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(392, 173);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.lvGeneBank});
			this.MinimumSize = new System.Drawing.Size(100, 64);
			this.Name = "CensusForm";
			this.ShowInTaskbar = false;
			this.SizeGripStyle = System.Windows.Forms.SizeGripStyle.Show;
			this.Text = "Census";
			this.ResumeLayout(false);

		}
		#endregion

		#region Drag-n-Drop
		private void lvGeneBank_ItemDrag(object sender, System.Windows.Forms.ItemDragEventArgs e)
		{
			this.DoDragDrop(e.Item,DragDropEffects.Move);
		}
		#endregion

		#region Sorting
//        private void lvGeneBank_ColumnClick(object sender, System.Windows.Forms.ColumnClickEventArgs e)
//		{
//			lvGeneBank.BeginUpdate();
//			if( sorter != null && sorter.col == e.Column )
//			{
//				sorter.desc = ! sorter.desc;
//				lvGeneBank.Sort();
//			}
//			else
//			{
//				sorter = new censusListSorter(e.Column);
//				this.lvGeneBank.ListViewItemSorter = sorter;
//			}
//			lvGeneBank.EndUpdate();
//		}
//
//		// Implements the manual sorting of items by columns.
//		private class censusListSorter : IComparer
//		{
//			public int col;
//			public bool desc = true;
//
//			public censusListSorter()
//			{
//				col = 0;
//			}
//			public censusListSorter(int column)
//			{
//				col = column;
//			}
//			public int Compare(object x, object y)
//			{
//				if( desc )
//				{
//					return String.Compare(((ListViewItem)y).SubItems[col].Text, ((ListViewItem)x).SubItems[col].Text);
//				}
//				else
//				{
//					return String.Compare(((ListViewItem)x).SubItems[col].Text, ((ListViewItem)y).SubItems[col].Text);
//				}
//			}
//		}
		#endregion

		#region Label Edit
		private void lvGeneBank_BeforeLabelEdit(object sender, System.Windows.Forms.LabelEditEventArgs e)
		{
			pauseUpdates = true;
			int curIndex = e.Item;
			if( curIndex < 0 || curIndex >= lvGeneBank.Items.Count )
			{
				pauseUpdates = false;
				return;
			}

			ListViewItem cur = lvGeneBank.Items[curIndex];
			//See if there is a ListViewItem being edited
			if( cur != null )
			{
				if( cur.Tag.GetType() == typeof(geneBankRecord) )
				{
					// allow label edit, after telling the Machine to stop cycling.
					ma.mustStopCycling = true;
					return;
				}
				else
				{
					// unknown tag found: don't allow label edit
					e.CancelEdit = true;
					pauseUpdates = false;
					return;
				}
			}
			else
			{
				// No ListViewItem found, prevent label edit
				e.CancelEdit = true;
				pauseUpdates = false;
				return;
			}
		}

		private void lvGeneBank_AfterLabelEdit(object sender, System.Windows.Forms.LabelEditEventArgs e)
		{
			pauseUpdates = true;
			int curIndex = e.Item;
			if( curIndex < 0 || curIndex >= lvGeneBank.Items.Count )
			{
				pauseUpdates = false;
				return;
			}

			ListViewItem cur = lvGeneBank.Items[curIndex];
			//See if there is a ListViewItem being edited
			if( cur != null )
			{
				if( cur.Tag.GetType() == typeof(geneBankRecord) )
				{
					if( e.Label == null || e.Label == "" )
					{
						e.CancelEdit = true;
						pauseUpdates = false;
						return;
					}
					geneBankRecord GBR = (geneBankRecord)(cur.Tag);
					// allow label edit
					string part1, part2;
					System.Text.RegularExpressions.Match mEdit = System.Text.RegularExpressions.Regex.Match(e.Label, "\\A\\d+/(.*)\\Z");
					
					part1 = GBR.mySize.ToString();
					if( mEdit.Success )
					{
						part2 = mEdit.Groups[1].Value;
					}
					else
					{
						part2 = e.Label;
					}

					string newName = part1 + '/' + part2;

					e.CancelEdit = true;

					if( this.ma.myGeneBank.renameSpecies(GBR.mySpeciesName,newName) )
					{
						species.Remove(cur.Text);
						species.Add(GBR.mySpeciesName,cur);
						cur.Text = GBR.mySpeciesName;
					}

					pauseUpdates = false;
					return;
				}
				else
				{
					// unknown tag found: don't allow label edit
					e.CancelEdit = true;
					pauseUpdates = false;
					return;
				}
			}
			else
			{
				// No ListViewItem found, prevent label edit
				e.CancelEdit = true;
				pauseUpdates = false;
				return;
			}
		}
		#endregion
	}
}
