using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

namespace ERGOT
{
    /// <summary>
	/// Configure Machine settings
	/// </summary>
	public class Configure : System.Windows.Forms.Form
	{
		private Machine ma;
		private MachineOptions curConfig;

		#region Form Element Declarations
		private System.Windows.Forms.PropertyGrid propgrid_Mao;
		private System.Windows.Forms.Button btnOK;
		private System.Windows.Forms.Button btnCancel;
		private System.Windows.Forms.Button btnApply;
		#endregion
		private System.Windows.Forms.Button btnDefaults;

		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public Configure()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			this.register( null );
			propgrid_Mao.PropertySort = PropertySort.Categorized;
		}

		#region register Machine
		public void register( Machine newMa )
		{
			this.ma = null;
			this.curConfig = null;
			propgrid_Mao.SelectedObject = null;
			this.btnOK.Enabled = false;
			this.btnApply.Enabled = false;

			if( newMa == null ) { return; }

			this.ma = newMa;
			this.curConfig = new MachineOptions(newMa);
			this.propgrid_Mao.SelectedObject = curConfig;
			this.btnOK.Enabled = true;
			this.btnApply.Enabled = true;
		}
		#endregion

		#region Dispose
		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}
		#endregion

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.propgrid_Mao = new System.Windows.Forms.PropertyGrid();
			this.btnOK = new System.Windows.Forms.Button();
			this.btnCancel = new System.Windows.Forms.Button();
			this.btnApply = new System.Windows.Forms.Button();
			this.btnDefaults = new System.Windows.Forms.Button();
			this.SuspendLayout();
			// 
			// propgrid_Mao
			// 
			this.propgrid_Mao.Anchor = (((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right);
			this.propgrid_Mao.CommandsVisibleIfAvailable = true;
			this.propgrid_Mao.LargeButtons = false;
			this.propgrid_Mao.LineColor = System.Drawing.SystemColors.ScrollBar;
			this.propgrid_Mao.Name = "propgrid_Mao";
			this.propgrid_Mao.PropertySort = System.Windows.Forms.PropertySort.NoSort;
			this.propgrid_Mao.Size = new System.Drawing.Size(536, 521);
			this.propgrid_Mao.TabIndex = 1;
			this.propgrid_Mao.Text = "propertyGrid1";
			this.propgrid_Mao.ViewBackColor = System.Drawing.SystemColors.Window;
			this.propgrid_Mao.ViewForeColor = System.Drawing.SystemColors.WindowText;
			// 
			// btnOK
			// 
			this.btnOK.Anchor = (System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right);
			this.btnOK.Location = new System.Drawing.Point(280, 529);
			this.btnOK.Name = "btnOK";
			this.btnOK.TabIndex = 2;
			this.btnOK.Text = "OK";
			this.btnOK.Click += new System.EventHandler(this.btnOK_Click);
			// 
			// btnCancel
			// 
			this.btnCancel.Anchor = (System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right);
			this.btnCancel.Location = new System.Drawing.Point(456, 529);
			this.btnCancel.Name = "btnCancel";
			this.btnCancel.TabIndex = 3;
			this.btnCancel.Text = "Cancel";
			this.btnCancel.Click += new System.EventHandler(this.btnCancel_Click);
			// 
			// btnApply
			// 
			this.btnApply.Anchor = (System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right);
			this.btnApply.Location = new System.Drawing.Point(368, 529);
			this.btnApply.Name = "btnApply";
			this.btnApply.TabIndex = 4;
			this.btnApply.Text = "Apply";
			this.btnApply.Click += new System.EventHandler(this.btnApply_Click);
			// 
			// btnDefaults
			// 
			this.btnDefaults.Anchor = (System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left);
			this.btnDefaults.Location = new System.Drawing.Point(8, 529);
			this.btnDefaults.Name = "btnDefaults";
			this.btnDefaults.Size = new System.Drawing.Size(96, 23);
			this.btnDefaults.TabIndex = 5;
			this.btnDefaults.Text = "Load Defaults";
			this.btnDefaults.Click += new System.EventHandler(this.btnDefaults_Click);
			// 
			// Configure
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(544, 557);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.btnDefaults,
																		  this.btnApply,
																		  this.btnCancel,
																		  this.btnOK,
																		  this.propgrid_Mao});
			this.Name = "Configure";
			this.Text = "Configure";
			this.ResumeLayout(false);

		}
		#endregion

		#region Button Click Event Handlers
		private void btnOK_Click(object sender, System.EventArgs e)
		{
			this.curConfig.ApplyOptions( ma );
			this.DialogResult = System.Windows.Forms.DialogResult.OK;
			this.Close();
		}

		private void btnApply_Click(object sender, System.EventArgs e)
		{
			this.curConfig.ApplyOptions( ma );
		}

		private void btnCancel_Click(object sender, System.EventArgs e)
		{
			this.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.Close();
		}

		private void btnDefaults_Click(object sender, System.EventArgs e)
		{
			this.curConfig = new MachineOptions();
			this.propgrid_Mao.SelectedObject = curConfig;
		}
		#endregion


	}
}
