//
// ERGOT : The Windows MacTierra clone
// copyright (c) 2005 plushpuffin@gmail.com
// http://www.wwddfd.com/
//
// latest build: http://www.wwddfd.com/frell/ergot.zip
//
// licensing terms still under consideration...

using System;
using System.Xml;

namespace ERGOT
{

	#region geneBankRecord
	/// <summary>
	/// A record of a genome and the corresponding species name.
	/// Also tracks the current living population and a count of the total
	/// number of creatures of this genome that have ever lived.
	/// </summary>
	public class geneBankRecord
	{
		public string myGenome;
		public short mySize = 0;
		public string mySpeciesName;
		public int myCurrentPopulation = 0;
		public int myCount = 0;
		public int myPeak = 0;
		public KTL.Node myNode = null;

		#region Constructors and Destructor
		public geneBankRecord()
		{} // do nothing

		/// <summary>
		/// Instantiate a geneBankRecord from an XML stream.
		/// </summary>
		/// <param name="x">The XmlTextReader object to use.</param>
		/// <param name="theMachine">The Gene Bank to which I will belong.</param>
		public geneBankRecord( System.Xml.XmlTextReader x, Machine theMachine, GeneBank theBank )
		{
			this.FromXMLStream( x, theMachine, theBank );
		}

		public geneBankRecord( Machine theMachine, int myPosition, short newSize, string newGenome, string newSpeciesName )
		{
			mySize = newSize;
			myGenome = newGenome;
			mySpeciesName = newSpeciesName;
			myCurrentPopulation = 1;
			myCount = 1;
			myPeak = 1;
		}

		public geneBankRecord initialize( Machine theMachine, int myPosition, short newSize, string newGenome, string newSpeciesName )
		{
			mySize = newSize;
			myGenome = newGenome;
			mySpeciesName = newSpeciesName;
			myCurrentPopulation = 1;
			myCount = 1;
			myPeak = 1;
			return this;
		}
		#endregion

		#region mutators
		public void inc()
		{
			myCurrentPopulation += 1;
			myCount += 1;
			if( myCurrentPopulation > myPeak )
			{
				myPeak = myCurrentPopulation;
			}
		}

		public void dec()
		{
			myCurrentPopulation -= 1;
		}
		#endregion
		
		#region FromXMLStream
		/// <summary>
		/// Import a geneBankRecord from an XML stream.
		/// </summary>
		/// <param name="x">The XmlTextReader to use.</param>
		public void FromXMLStream( XmlTextReader x, Machine theMachine, GeneBank theBank )
		{
			while( x.Name == "" )
			{
				x.Read();
			}
			if( x.Name.ToLower() != "genebankrecord" )
			{
				// not the correct type of grouping.
				return;
			}
			string nodeName = "";
			bool keepReading = true;
			bool readSuccess = true;

			while( keepReading && x.Read() )
			{
				switch (x.NodeType)
				{
					case XmlNodeType.Element:
						nodeName = x.Name.ToLower();
						break;
					case XmlNodeType.Text:
						switch (nodeName)
						{
							case "mysize":
								try
								{
									mySize = System.Int16.Parse( x.Value, theMachine.intStyle );
								}
								catch (Exception e)
								{
									readSuccess = false;
									theMachine.ioError(nodeName,e);
								}
								break;
							case "myspecies":
								mySpeciesName = x.Value;
								break;
							case "mygenome":
								myGenome = x.Value;
								break;
							case "mycurrentpopulation":
								// we ignore this, because it's up to the ergots
								// to determine their current population
								break;
							case "mypeak":
								try
								{
									myPeak = System.Int32.Parse( x.Value, theMachine.intStyle );
								}
								catch (Exception e)
								{
									theMachine.ioError(nodeName,e);
								}
								break;
							case "mytotallivinganddead":
								try
								{
									myCount = System.Int32.Parse( x.Value, theMachine.intStyle );
								}
								catch (Exception e)
								{
									theMachine.ioError(nodeName,e);
								}
								break;
							default:
								break;
						} // switch
						break;
					case XmlNodeType.EndElement:
						if( x.Name.ToLower() == "genebankrecord" )
						{
							keepReading = false;
							break;
						}
						nodeName = "";
						break;
					default:
						break;
				} // switch
			} // while
			if( readSuccess )
			{
				theBank[myGenome] = this;
			}
			else
			{
				theMachine.ioError( myGenome, new Exception("failure"));
			}
		} // function
		#endregion

		#region ToXMLStream
		/// <summary>
		/// Output this bank record to the specified XML writer stream.
		/// </summary>
		/// <param name="x">An XmlTextWriter object.</param>
		public void ToXMLStream( XmlTextWriter x )
		{
			x.WriteStartElement("geneBankRecord");
			if(true)
			{
				x.WriteElementString( "mySpecies", this.mySpeciesName );
				x.WriteElementString( "mySize", this.mySize.ToString() );
				x.WriteElementString( "myCurrentPopulation", this.myCurrentPopulation.ToString() );
				x.WriteElementString( "myPeak", this.myPeak.ToString() );
				x.WriteElementString( "myTotalLivingAndDead", this.myCount.ToString() );
				//				x.WriteStartElement("moniker");
				//				{
				//					x.WriteBinHex( this.myAssembly, 0, mySize );
				//				}
				//x.WriteEndElement();
				x.WriteElementString( "myGenome", this.myGenome );
			}
			x.WriteEndElement();
		}
		#endregion

	}
	#endregion

	#region GeneBank
	/// <summary>
	/// Summary description for GeneLibrary.
	/// </summary>
	public class GeneBank
	{
		#region Member Variables

		internal int recycleStack_TopIndex = -1;
		internal geneBankRecord[] recycleStack = null;

		internal System.Collections.Hashtable _specdex = null;
		internal System.Collections.Hashtable _genedex = null;
		internal KTL.dList _genebank = null;
		#endregion

		#region Constructors
		public GeneBank()
		{
			_genebank = new KTL.dList();
			_genedex = new System.Collections.Hashtable( 313 );
			_specdex = new System.Collections.Hashtable( 313 );
			recycleStack = new geneBankRecord[64];
		}
		#endregion

		#region accessors
		public int Count
		{
			get
			{
				return _genebank.Count;
			}
		}

		public bool ContainsSpecies(string theSpeciesName)
		{
			return _specdex.Contains(theSpeciesName);
		}

		public geneBankRecord getSpeciesRecord(string theSpeciesName)
		{
			KTL.Node cur = (KTL.Node)(_specdex[theSpeciesName]);
			if( cur == null )
			{
				return null;
			}
			else
			{
				return (geneBankRecord)(cur.Value);
			}
		}
		#endregion

		#region mutators
		public geneBankRecord this[string genomeString] 
		{
			get
			{
				KTL.Node cur = (KTL.Node)(_genedex[genomeString]);
				if( cur == null )
				{
					return null;
				}
				else
				{
					return (geneBankRecord)cur.Value;
				}
			}
			
			set
			{
				KTL.Node cur = (KTL.Node)(_genedex[genomeString]);
				if( cur != null )
				{
					// remove whatever we are replacing.
					// This should never actually happen.
					System.Windows.Forms.MessageBox.Show("A geneBankRecord is being replaced!", "GeneBankRecord", System.Windows.Forms.MessageBoxButtons.OK, System.Windows.Forms.MessageBoxIcon.Error);
					this.Remove(genomeString);
				}
				cur = value.myNode;
				if( cur == null )
				{
					cur = new KTL.Node(value);
					value.myNode = cur;
				}
				_genedex[genomeString] = cur;
				_specdex[value.mySpeciesName] = cur;
				_genebank.push( cur );
			}
		}

		public bool renameSpecies( string oldSpeciesName, string newSpeciesName )
		{
			if( this.ContainsSpecies( newSpeciesName ) ) { return false; }
			KTL.Node cur = (KTL.Node)(this._specdex[oldSpeciesName]);
			if( cur == null ) { return false; }
			geneBankRecord GBR = (geneBankRecord)cur.Value;
			if( GBR == null ) { return false; }
			this._specdex.Remove(oldSpeciesName);
			GBR.mySpeciesName = newSpeciesName;
			this._specdex[newSpeciesName] = cur;
			return true;
		}
		#endregion

		#region Recycling
		public void Remove(string genomeString)
		{
			KTL.Node cur = (KTL.Node)(_genedex[genomeString]);
			if( cur == null )
			{
				return;
			}
			_specdex.Remove(this[genomeString].mySpeciesName);
			_genedex.Remove(genomeString);
			_genebank.removeNode(cur);
			cur.NextNode = null;
			cur.PrevNode = null;
			if( recycleStack_TopIndex < 63 )
			{
				++recycleStack_TopIndex;
				recycleStack[recycleStack_TopIndex] = (geneBankRecord)cur.Value;
			}
		}

		public geneBankRecord NewGeneBankRecord()
		{
			geneBankRecord rec;
			if( recycleStack_TopIndex == -1 )
			{
				// nothing in the resurrectionStack. We must create a new ergot.
				rec = new geneBankRecord();
			}
			else
			{
				// we can re-use an ergot from the resurrectionStack.
				rec = recycleStack[recycleStack_TopIndex];
				recycleStack[recycleStack_TopIndex] = null;
				--recycleStack_TopIndex;
			}
			return rec;
		}
		#endregion
		
		#region FromXMLStream
		public void FromXMLStream( XmlTextReader x, Machine theMachine )
		{
			while( x.Name == "" )
			{
				x.Read();
			}
			if( x.Name.ToLower() != "genebank" )
			{
				// not the correct type of grouping.
				return;
			}
			string nodeName = "";

			while( x.Read() )
			{
				switch (x.NodeType)
				{
					case XmlNodeType.Element:
						nodeName = x.Name.ToLower();
						if( nodeName == "genebankrecord" )
						{
							geneBankRecord cur = new geneBankRecord(x, theMachine, this);
						}
						break;
					case XmlNodeType.Text:
						break;
					case XmlNodeType.EndElement:
					switch (x.Name.ToLower() )
					{
						case "genebank":
							return;
						default:
							break;
					}
						nodeName = "";
						break;
					default:
						break;
				} // switch
			} // while
		}
		#endregion

		#region ToXMLStream
		/// <summary>
		/// Output the entire contents of this GeneBank, and its embryonic daughter,
		/// to the specified XML writer stream.
		/// </summary>
		/// <param name="x">An XmlTextWriter object.</param>
		public void ToXMLStream( System.Xml.XmlTextWriter x )
		{
			x.WriteStartElement( "GeneBank" );
			if(true)
			{
				KTL.Node cur = this._genebank.Head;
				while( cur != null && ! cur.isNull )
				{
					((geneBankRecord)(cur.Value)).ToXMLStream(x);
					cur = cur.NextNode;
				}
			} // GeneBank
			x.WriteEndElement();
		}
		#endregion

	}
	#endregion

}
