//
// ERGOT : The Windows MacTierra clone
// copyright (c) 2005 plushpuffin@gmail.com
// http://www.wwddfd.com/
//
// latest build: http://www.wwddfd.com/frell/ergot.zip
//
// licensing terms still under consideration...

using System;
using System.Xml;

namespace ERGOT
{

	#region geneLibRecord
	public class geneLibRecord
	{
		public string myGenome;
		public short mySize = 0;
		public byte[] myAssembly = null;
		public string mySpeciesName;
		public string myDescription;
		public string myCreator;

		#region Constructors
		/// <summary>
		/// Instantiate a geneLibRecord from an XML stream.
		/// </summary>
		/// <param name="x">The XmlTextReader object to use.</param>
		/// <param name="theMachine">The Gene Library to which I will belong.</param>
		public geneLibRecord( System.Xml.XmlTextReader x, GeneLib theLib )
		{
			this.FromXMLStream( x, theLib );
		}

		public geneLibRecord( string newGenome, string newSpeciesName )
		{
			mySize = (short)(newGenome.Length / 2 );
			myGenome = newGenome;
			mySpeciesName = newSpeciesName;
			myAssembly = new byte[mySize];
			for( int i = 0; i < mySize; ++i )
			{
				string sb = newGenome.Substring(i<<1,2);
				byte cb = System.Convert.ToByte(sb,16);
				myAssembly[i] = cb;
			}
		}

		public geneLibRecord( byte[] soup, int soupSize, int myPosition, short newSize, string newGenome, string newSpeciesName )
		{
			mySize = newSize;
			myGenome = newGenome;
			mySpeciesName = newSpeciesName;
			myAssembly = new Byte[mySize];
			int pos = myPosition % soupSize;
			for( int i = 0; i < mySize; ++i )
			{
				myAssembly[i] = soup[pos];
				pos = (pos+1)%soupSize;
			}
		}

		public geneLibRecord( byte[] newAssembly, short newSize, string newGenome, string newSpeciesName )
		{
			mySize = newSize;
			myGenome = newGenome;
			mySpeciesName = newSpeciesName;
			myAssembly = newAssembly;
		}
		#endregion

		#region accessors
		public int myPopulation( Machine theMachine )
		{
			geneBankRecord cur = (geneBankRecord)(theMachine.myGeneBank[myGenome]);
			if( cur == null )
			{
				return 0;
			}
			return cur.myCurrentPopulation;
		}

		public string Description
		{
			get
			{
				return myDescription;
			}
			set
			{
				myDescription = value;
			}
		}

		public string Creator
		{
			get
			{
				return myCreator;
			}
			set
			{
				myCreator = value;
			}
		}
		#endregion

		#region FromXMLStream
		/// <summary>
		/// Import a geneLibRecord from an XML stream.
		/// </summary>
		/// <param name="x">The XmlTextReader to use.</param>
		public void FromXMLStream( XmlTextReader x, GeneLib theLib )
		{
			while( x.Name == "" )
			{
				x.Read();
			}
			if( x.Name.ToLower() != "genelibrecord" )
			{
				// not the correct type of grouping.
				return;
			}
			string nodeName = "";
			bool keepReading = true;
			bool readSuccess = true;

			while( keepReading && x.Read() )
			{
				switch (x.NodeType)
				{
					case XmlNodeType.Element:
						nodeName = x.Name.ToLower();
					switch (nodeName)
					{
						case "moniker":
							try
							{
								myAssembly = new byte[mySize];
								x.ReadBinHex(myAssembly, 0, mySize );
								System.Text.StringBuilder myGenomeSB = new System.Text.StringBuilder(mySize<<1,mySize<<1);
								for( short i = 0; i < mySize; ++i )
								{
									myGenomeSB.Append(myAssembly[i].ToString("X2"));
								}
								myGenome = myGenomeSB.ToString();
							}
							catch (Exception e)
							{
								theLib.ioError(nodeName,e);
							}
							break;
						default:
							break;
					}
						break;
					case XmlNodeType.Text:
					switch (nodeName)
					{
						case "mysize":
							try
							{
								mySize = System.Int16.Parse( x.Value, theLib.intStyle );
							}
							catch (Exception e)
							{
								readSuccess = false;
								theLib.ioError(nodeName,e);
							}
							break;
						case "myspecies":
							mySpeciesName = x.Value;
							break;
						case "mycreator":
							myCreator = x.Value;
							break;
						case "mydescription":
							myDescription = x.Value;
							break;
						default:
							break;
					}
						break;
					case XmlNodeType.EndElement:
						if( x.Name.ToLower() == "genelibrecord" )
						{
							keepReading = false;
							break;
						}
						nodeName = "";
						break;
					default:
						break;
				} // switch
			} // while
			if( readSuccess )
			{
				theLib[myGenome] = this;
			}
			else
			{
				theLib.ioError( myGenome, new Exception("failure"));
			}
		} // function
		#endregion

		#region ToXMLStream
		/// <summary>
		/// Output this library record to the specified XML writer stream.
		/// </summary>
		/// <param name="x">An XmlTextWriter object.</param>
		public void ToXMLStream( XmlTextWriter x, GeneLib myLib )
		{
			x.WriteStartElement("geneLibRecord");
			if(true)
			{
				x.WriteElementString( "mySpecies", this.mySpeciesName );
				x.WriteElementString( "mySize", this.mySize.ToString() );
				x.WriteElementString( "myCreator", this.myCreator );
				x.WriteElementString( "myDescription", this.myDescription );
				x.WriteStartElement("moniker");
				if(true)
				{
					x.WriteBinHex( this.myAssembly, 0, mySize );
				}
				x.WriteEndElement();
				if(true)
				{
					// DEBUG: GENOME PRINTING
					x.WriteStartElement( "genome" );
					for( int i = 0; i < mySize; ++i )
					{
						x.WriteStartElement( "cell" );
						x.WriteAttributeString( "", "index", null, i.ToString("D3") );
						x.WriteAttributeString( "", "dec", null, myAssembly[i].ToString("D2") );
						x.WriteAttributeString( "", "hex", null, myAssembly[i].ToString("X2") );
						x.WriteString(myLib.Instructions[myAssembly[i]]);
						x.WriteEndElement();
					}
					x.WriteEndElement();
				}
			}
			x.WriteEndElement();
		}
		#endregion

	}
	#endregion

	#region GeneLib
	/// <summary>
	/// Summary description for GeneLib.
	/// </summary>
	public class GeneLib
	{
		#region Member Variables
		internal System.Collections.Hashtable _genedex = null;
		internal System.Collections.ArrayList _genelib = null;
		private int _errorCode;

		private string[] instrs;
		
		private string _xmlFileName;
		private System.Text.StringBuilder _errorString;

		public readonly System.Globalization.NumberStyles sharedStyle;
		public readonly System.Globalization.NumberStyles intStyle;
		public readonly System.Globalization.NumberStyles hexStyle;
		public readonly System.Globalization.NumberStyles dblStyle;
		public readonly System.Globalization.NumberStyles bytStyle;
		#endregion

		#region Constructors
		public GeneLib()
		{
			sharedStyle = System.Globalization.NumberStyles.AllowLeadingWhite | System.Globalization.NumberStyles.AllowTrailingWhite;
			intStyle = sharedStyle | System.Globalization.NumberStyles.AllowThousands | System.Globalization.NumberStyles.AllowLeadingSign;
			hexStyle = sharedStyle | System.Globalization.NumberStyles.AllowHexSpecifier;
			dblStyle = sharedStyle | System.Globalization.NumberStyles.AllowThousands | System.Globalization.NumberStyles.AllowLeadingSign | System.Globalization.NumberStyles.AllowDecimalPoint | System.Globalization.NumberStyles.AllowExponent;
			bytStyle = sharedStyle | System.Globalization.NumberStyles.AllowLeadingSign;
			_errorString = new System.Text.StringBuilder(2001);
			_genelib = new System.Collections.ArrayList( 16 );
			_genedex = new System.Collections.Hashtable( 37 );
		}
		#endregion

		#region accessors
		public int Count
		{
			get
			{
				return _genelib.Count;
			}
		}

		public bool isValid
		{
			get
			{
				return (_errorCode < 1) && (_genelib != null);
			}
		}

		public int ErrorCode
		{
			get 
			{
				return _errorCode;
			}
		}

		public string ErrorString
		{
			get
			{
				return _errorString.ToString();
			}
		}
		#endregion

		#region mutators
		public geneLibRecord this[string genomeString] 
		{
			get
			{
				object pos = _genedex[genomeString];
				return pos == null ? null : (geneLibRecord)(_genelib[(int)pos]);
			} 
			
			set
			{
				if( _genedex.ContainsKey(genomeString) )
				{
					_genelib[(int)(_genedex[genomeString])] = value;
				}
				else
				{
					_genedex[genomeString] = _genelib.Count;
					_genelib.Add(value);
				}
			}
		}

		public string[] Instructions
		{
			get
			{
				return instrs;
			}
			set
			{
				instrs = value;
			}
		}


		public void delete(string genomeString)
		{
			_genelib.RemoveAt((int)(_genedex[genomeString]));
			_genedex.Remove(genomeString);
		}
		#endregion

		#region ioError
		public void ioError( string theName, Exception e )
		{
			_errorString.Append( "XML I/O encountered an error at " + theName + ": " + e.Message +"\n" );
			_errorCode = 10;
		}
		public void saveError( string theName, Exception e )
		{
			_errorString.Append( "XML Output encountered an error at " + theName + ": " + e.Message +"\n" );
			_errorCode = -10;
		}
		#endregion
		
		#region FromXMLFile
		public void FromXMLFile( string xmlFileName )
		{
			XmlTextReader x = null;
			try
			{
				_errorCode = 0;
				_errorString = new System.Text.StringBuilder(2001);
				x = new XmlTextReader(xmlFileName);
				while( x.Name == "" )
				{
					x.Read();
				}

				// save the file name for posterity.
				_xmlFileName = xmlFileName;

				if( x.Name.ToLower() != "xml" )
				{
					// not an XML document.
					_errorCode = 2;
					_errorString.Append( "File " + xmlFileName + " was not a valid XML file.\n" );
					return;
				}
				while( x.Read() )
				{
					if( x.NodeType == XmlNodeType.Element && x.Name.ToLower() == "genelib" )
					{
						break;
					}
				}
				if( x.EOF )
				{
					// not the correct type of XML document.
					_errorCode = 2;
					_errorString.Append( "File " + xmlFileName + " was empty, or was not a valid XML::GeneLib file.\n" );
					return;
				}

				while( x.Read() )
				{
					switch(x.NodeType)
					{
						case XmlNodeType.Element:
						switch(x.Name.ToLower())
						{
							case "genelibrecord":
								geneLibRecord curRecord = new geneLibRecord(x, this);
								break;
							default:
								break;
						}
							break;
						case XmlNodeType.Text:
							break;
						case XmlNodeType.EndElement:
							break;
						default:
							break;
					}
				}
				x.Close();
			}
			catch (Exception ex)
			{
				ioError(xmlFileName, ex);
				try
				{
					if( x != null )
					{
						x.Close();
					}
				}
				catch
				{
					;
				}
			}
		}

		#endregion

		#region ToXMLFile
		/// <summary>
		/// Output this GeneLib to the specified file.
		/// </summary>
		/// <param name="xmlFileName">The file name to use. This file will be overwritten if it exists.</param>
		public void ToXMLFile( string xmlFileName )
		{
			XmlTextWriter x = null;
			try
			{
				x = new XmlTextWriter(xmlFileName, null);
				x.Formatting = System.Xml.Formatting.Indented;
				x.WriteStartDocument();
			
				x.WriteStartElement( "GeneLib" );
				if(true)
				{
					foreach (geneLibRecord cur in this._genelib)
					{
						cur.ToXMLStream(x, this);
					}
				} // GeneLib
				x.WriteEndElement();
				x.WriteEndDocument();
				x.Close();
			}
			catch (Exception ex)
			{
				saveError(xmlFileName, ex);
				try
				{
					if( x != null )
					{
						x.Close();
					}
				}
				catch
				{
					;
				}
			}
		}
		#endregion

	}
	#endregion

}
