//
// ERGOT : The Windows MacTierra clone
// copyright (c) 2005 plushpuffin@gmail.com
// http://www.wwddfd.com/
//
// latest build: http://www.wwddfd.com/frell/ergot.zip
//
// licensing terms still under consideration...

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Threading;


namespace ERGOT
{
	/// <summary>
	/// WinTierra is the graphical front-end for the ERGOT.Machine.
	/// </summary>
	public class WinTierra : System.Windows.Forms.Form
	{
		
		private Machine ma;
		private Thread tre;
		private Bitmap bmpSoup;
		private Graphics gSoup;
		private Win32.HiPerfTimer myTimer = null;
		System.Globalization.NumberFormatInfo nfi = null;
		private string myFileName = "";
		private LibraryForm libf = null;
		private CensusForm censusf = null;
		private ulong prevTicks = 0;
        private ulong prevWork = 0;
		//private string debugLog = "";

		#region Form Elements
		private System.Drawing.Rectangle rectSoup;
		public System.Windows.Forms.PictureBox soup;
		private System.Windows.Forms.MainMenu mainMenu1;
		private System.ComponentModel.IContainer components;
		private System.Windows.Forms.Label lbl_ip;
		private System.Windows.Forms.Label lbl_ax;
		private System.Windows.Forms.Label lbl_bx;
		private System.Windows.Forms.Label lbl_cx;
		private System.Windows.Forms.Label lbl_dx;
		private System.Windows.Forms.Label lbl_sp;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.Label label3;
		private System.Windows.Forms.Label label4;
		private System.Windows.Forms.Label label5;
		private System.Windows.Forms.Label label6;
		private System.Windows.Forms.GroupBox grp_registers;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.Label label8;
		private System.Windows.Forms.Label lbl_inst;
		private System.Windows.Forms.GroupBox groupBox1;
		private System.Windows.Forms.Label label7;
		private System.Windows.Forms.Label label10;
		private System.Windows.Forms.Label label11;
		private System.Windows.Forms.Label label18;
		private System.Windows.Forms.Label label19;
		private System.Windows.Forms.Label label20;
		private System.Windows.Forms.Label label21;
		private System.Windows.Forms.Label lbl_pinst;
		private System.Windows.Forms.Label lbl_psp;
		private System.Windows.Forms.Label lbl_pdx;
		private System.Windows.Forms.Label lbl_pcx;
		private System.Windows.Forms.Label lbl_pbx;
		private System.Windows.Forms.Label lbl_pax;
		private System.Windows.Forms.Label lbl_pip;
		private System.Windows.Forms.Label lbl_pfl;
		private System.Windows.Forms.Label label12;
		private System.Windows.Forms.Label lbl_fl;
		private System.Windows.Forms.Label label13;
		private System.Windows.Forms.StatusBar sb_mainWin;
		private System.Windows.Forms.Button btn_Step;
		private System.Windows.Forms.Button btn_Go;
		private System.Windows.Forms.ImageList imageList1;
		private System.Windows.Forms.Panel picPanel;
		private System.Windows.Forms.MenuItem menu_View;
		private System.Windows.Forms.MenuItem mView_Normal;
		private System.Windows.Forms.MenuItem mView_AdultEmbryo;
		private System.Windows.Forms.MenuItem menuItem8;
		private System.Windows.Forms.MenuItem menuItem9;
		private System.Windows.Forms.MenuItem menuItem10;
		private System.Windows.Forms.MenuItem menu_File;
		private System.Windows.Forms.MenuItem menu_Edit;
		private System.Windows.Forms.MenuItem mFile_New;
		private System.Windows.Forms.MenuItem mFile_Open;
		private System.Windows.Forms.MenuItem mFile_Save;
		private System.Windows.Forms.MenuItem mFile_SaveAs;
		private System.Windows.Forms.MenuItem mFile_Exit;
		private System.Windows.Forms.SaveFileDialog dlg_SaveXML;
		private System.Windows.Forms.OpenFileDialog dlg_OpenXML;
		private System.Windows.Forms.StatusBarPanel sbp_Threads;
		private System.Windows.Forms.StatusBarPanel sbp_NumSpecies;
		private System.Windows.Forms.StatusBarPanel sbp_Load;
		private System.Windows.Forms.StatusBarPanel sbp_Speed;
		private System.Windows.Forms.StatusBarPanel sbp_AvgSize;
		private System.Windows.Forms.StatusBarPanel sbp_Cycles;
		private System.Windows.Forms.MenuItem menuItem1;
		private System.Windows.Forms.MenuItem mView_GeneLibrary;
		private System.Windows.Forms.MenuItem mView_Census;
		private System.Windows.Forms.MenuItem menu_Help;
		private System.Windows.Forms.MenuItem mHelp_About;
		private System.Windows.Forms.StatusBarPanel sbp_Work;
		private System.Windows.Forms.StatusBarPanel sbp_Ticks;
		private System.Windows.Forms.MenuItem mEdit_Configure;
		private System.Windows.Forms.Button btn_Dump;
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			WinTierra wa = new WinTierra();
			Machine newMa = new Machine( System.IO.Directory.GetCurrentDirectory() + "\\default.xml" );
			if( newMa.isValid )
			{
				wa.register( newMa );
				Application.Run(wa);
			}
			else
			{
				// display error.
				//MessageBox.Show(newMa.ErrorString,"Machine Initialization Error",System.Windows.Forms.MessageBoxButtons.OK,System.Windows.Forms.MessageBoxIcon.Error);
				newMa = new Machine();
				newMa.initialize();
				wa.register( newMa );
				Application.Run(wa);
			}
		}

		#region WinTierra Constructor
		public WinTierra()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();
			myTimer = new Win32.HiPerfTimer();
		
			nfi = new System.Globalization.CultureInfo("en-US", false).NumberFormat;
			nfi.NumberGroupSeparator = ",";
			nfi.NumberDecimalDigits = 0;
			nfi.PercentDecimalDigits = 1;
			
			((System.Windows.Forms.Control)soup).AllowDrop = true;
		}
		#endregion
		
		#region Dispose
		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( ma != null )
			{
				this.register(null);
			}
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			if( bmpSoup != null )
			{
				bmpSoup.Dispose();
			}
			if( gSoup != null )
			{
				gSoup.Dispose();
			}
			if( myTimer != null )
			{
				myTimer.Stop();
			}
			base.Dispose( disposing );
		}
		#endregion

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(WinTierra));
			this.soup = new System.Windows.Forms.PictureBox();
			this.mainMenu1 = new System.Windows.Forms.MainMenu();
			this.menu_File = new System.Windows.Forms.MenuItem();
			this.mFile_New = new System.Windows.Forms.MenuItem();
			this.menuItem8 = new System.Windows.Forms.MenuItem();
			this.mFile_Open = new System.Windows.Forms.MenuItem();
			this.menuItem9 = new System.Windows.Forms.MenuItem();
			this.mFile_Save = new System.Windows.Forms.MenuItem();
			this.mFile_SaveAs = new System.Windows.Forms.MenuItem();
			this.menuItem10 = new System.Windows.Forms.MenuItem();
			this.mFile_Exit = new System.Windows.Forms.MenuItem();
			this.menu_Edit = new System.Windows.Forms.MenuItem();
			this.menu_View = new System.Windows.Forms.MenuItem();
			this.mView_Normal = new System.Windows.Forms.MenuItem();
			this.mView_AdultEmbryo = new System.Windows.Forms.MenuItem();
			this.menuItem1 = new System.Windows.Forms.MenuItem();
			this.mView_Census = new System.Windows.Forms.MenuItem();
			this.mView_GeneLibrary = new System.Windows.Forms.MenuItem();
			this.menu_Help = new System.Windows.Forms.MenuItem();
			this.mHelp_About = new System.Windows.Forms.MenuItem();
			this.picPanel = new System.Windows.Forms.Panel();
			this.lbl_ip = new System.Windows.Forms.Label();
			this.sb_mainWin = new System.Windows.Forms.StatusBar();
			this.sbp_Cycles = new System.Windows.Forms.StatusBarPanel();
			this.sbp_Ticks = new System.Windows.Forms.StatusBarPanel();
			this.sbp_Speed = new System.Windows.Forms.StatusBarPanel();
			this.sbp_Work = new System.Windows.Forms.StatusBarPanel();
			this.sbp_Load = new System.Windows.Forms.StatusBarPanel();
			this.sbp_NumSpecies = new System.Windows.Forms.StatusBarPanel();
			this.sbp_Threads = new System.Windows.Forms.StatusBarPanel();
			this.sbp_AvgSize = new System.Windows.Forms.StatusBarPanel();
			this.btn_Step = new System.Windows.Forms.Button();
			this.imageList1 = new System.Windows.Forms.ImageList(this.components);
			this.grp_registers = new System.Windows.Forms.GroupBox();
			this.label13 = new System.Windows.Forms.Label();
			this.lbl_fl = new System.Windows.Forms.Label();
			this.label8 = new System.Windows.Forms.Label();
			this.lbl_inst = new System.Windows.Forms.Label();
			this.label6 = new System.Windows.Forms.Label();
			this.label5 = new System.Windows.Forms.Label();
			this.lbl_sp = new System.Windows.Forms.Label();
			this.lbl_dx = new System.Windows.Forms.Label();
			this.lbl_cx = new System.Windows.Forms.Label();
			this.lbl_bx = new System.Windows.Forms.Label();
			this.lbl_ax = new System.Windows.Forms.Label();
			this.label1 = new System.Windows.Forms.Label();
			this.label2 = new System.Windows.Forms.Label();
			this.label3 = new System.Windows.Forms.Label();
			this.label4 = new System.Windows.Forms.Label();
			this.groupBox1 = new System.Windows.Forms.GroupBox();
			this.label12 = new System.Windows.Forms.Label();
			this.lbl_pfl = new System.Windows.Forms.Label();
			this.label7 = new System.Windows.Forms.Label();
			this.lbl_pinst = new System.Windows.Forms.Label();
			this.label10 = new System.Windows.Forms.Label();
			this.label11 = new System.Windows.Forms.Label();
			this.lbl_psp = new System.Windows.Forms.Label();
			this.lbl_pdx = new System.Windows.Forms.Label();
			this.lbl_pcx = new System.Windows.Forms.Label();
			this.lbl_pbx = new System.Windows.Forms.Label();
			this.lbl_pax = new System.Windows.Forms.Label();
			this.lbl_pip = new System.Windows.Forms.Label();
			this.label18 = new System.Windows.Forms.Label();
			this.label19 = new System.Windows.Forms.Label();
			this.label20 = new System.Windows.Forms.Label();
			this.label21 = new System.Windows.Forms.Label();
			this.btn_Go = new System.Windows.Forms.Button();
			this.btn_Dump = new System.Windows.Forms.Button();
			this.dlg_SaveXML = new System.Windows.Forms.SaveFileDialog();
			this.dlg_OpenXML = new System.Windows.Forms.OpenFileDialog();
			this.mEdit_Configure = new System.Windows.Forms.MenuItem();
			this.picPanel.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.sbp_Cycles)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.sbp_Ticks)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.sbp_Speed)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.sbp_Work)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.sbp_Load)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.sbp_NumSpecies)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.sbp_Threads)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.sbp_AvgSize)).BeginInit();
			this.grp_registers.SuspendLayout();
			this.groupBox1.SuspendLayout();
			this.SuspendLayout();
			// 
			// soup
			// 
			this.soup.Anchor = (((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right);
			this.soup.BackColor = System.Drawing.SystemColors.Control;
			this.soup.Location = new System.Drawing.Point(10, 10);
			this.soup.Name = "soup";
			this.soup.Size = new System.Drawing.Size(512, 512);
			this.soup.TabIndex = 0;
			this.soup.TabStop = false;
			this.soup.DragEnter += new System.Windows.Forms.DragEventHandler(this.soup_DragEnter);
			this.soup.DragDrop += new System.Windows.Forms.DragEventHandler(this.soup_DragDrop);
			// 
			// mainMenu1
			// 
			this.mainMenu1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.menu_File,
																					  this.menu_Edit,
																					  this.menu_View,
																					  this.menu_Help});
			// 
			// menu_File
			// 
			this.menu_File.Index = 0;
			this.menu_File.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.mFile_New,
																					  this.menuItem8,
																					  this.mFile_Open,
																					  this.menuItem9,
																					  this.mFile_Save,
																					  this.mFile_SaveAs,
																					  this.menuItem10,
																					  this.mFile_Exit});
			this.menu_File.Text = "File";
			// 
			// mFile_New
			// 
			this.mFile_New.Index = 0;
			this.mFile_New.Shortcut = System.Windows.Forms.Shortcut.CtrlN;
			this.mFile_New.Text = "New";
			this.mFile_New.Click += new System.EventHandler(this.mFile_New_Click);
			// 
			// menuItem8
			// 
			this.menuItem8.Index = 1;
			this.menuItem8.Text = "-";
			// 
			// mFile_Open
			// 
			this.mFile_Open.Index = 2;
			this.mFile_Open.Shortcut = System.Windows.Forms.Shortcut.CtrlO;
			this.mFile_Open.Text = "Open";
			this.mFile_Open.Click += new System.EventHandler(this.mFile_Open_Click);
			// 
			// menuItem9
			// 
			this.menuItem9.Index = 3;
			this.menuItem9.Text = "-";
			// 
			// mFile_Save
			// 
			this.mFile_Save.Index = 4;
			this.mFile_Save.Shortcut = System.Windows.Forms.Shortcut.CtrlS;
			this.mFile_Save.Text = "Save";
			this.mFile_Save.Click += new System.EventHandler(this.mFile_Save_Click);
			// 
			// mFile_SaveAs
			// 
			this.mFile_SaveAs.Index = 5;
			this.mFile_SaveAs.Shortcut = System.Windows.Forms.Shortcut.CtrlShiftS;
			this.mFile_SaveAs.Text = "Save As";
			this.mFile_SaveAs.Click += new System.EventHandler(this.mFile_SaveAs_Click);
			// 
			// menuItem10
			// 
			this.menuItem10.Index = 6;
			this.menuItem10.Text = "-";
			// 
			// mFile_Exit
			// 
			this.mFile_Exit.Index = 7;
			this.mFile_Exit.Shortcut = System.Windows.Forms.Shortcut.CtrlX;
			this.mFile_Exit.Text = "Exit";
			this.mFile_Exit.Click += new System.EventHandler(this.mFile_Exit_Click);
			// 
			// menu_Edit
			// 
			this.menu_Edit.Index = 1;
			this.menu_Edit.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.mEdit_Configure});
			this.menu_Edit.Text = "Edit";
			// 
			// menu_View
			// 
			this.menu_View.Index = 2;
			this.menu_View.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.mView_Normal,
																					  this.mView_AdultEmbryo,
																					  this.menuItem1,
																					  this.mView_Census,
																					  this.mView_GeneLibrary});
			this.menu_View.Text = "View";
			// 
			// mView_Normal
			// 
			this.mView_Normal.Checked = true;
			this.mView_Normal.Index = 0;
			this.mView_Normal.RadioCheck = true;
			this.mView_Normal.Shortcut = System.Windows.Forms.Shortcut.CtrlD;
			this.mView_Normal.Text = "Normal";
			this.mView_Normal.Click += new System.EventHandler(this.mView_Normal_Click);
			// 
			// mView_AdultEmbryo
			// 
			this.mView_AdultEmbryo.Index = 1;
			this.mView_AdultEmbryo.RadioCheck = true;
			this.mView_AdultEmbryo.Shortcut = System.Windows.Forms.Shortcut.CtrlD;
			this.mView_AdultEmbryo.Text = "Adults/Embryos";
			this.mView_AdultEmbryo.Click += new System.EventHandler(this.mView_AdultEmbryo_Click);
			// 
			// menuItem1
			// 
			this.menuItem1.Index = 2;
			this.menuItem1.Text = "-";
			// 
			// mView_Census
			// 
			this.mView_Census.Index = 3;
			this.mView_Census.Shortcut = System.Windows.Forms.Shortcut.CtrlE;
			this.mView_Census.Text = "Census";
			this.mView_Census.Click += new System.EventHandler(this.mView_Census_Click);
			// 
			// mView_GeneLibrary
			// 
			this.mView_GeneLibrary.Index = 4;
			this.mView_GeneLibrary.Shortcut = System.Windows.Forms.Shortcut.CtrlL;
			this.mView_GeneLibrary.Text = "Gene Library";
			this.mView_GeneLibrary.Click += new System.EventHandler(this.mView_GeneLibrary_Click);
			// 
			// menu_Help
			// 
			this.menu_Help.Index = 3;
			this.menu_Help.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.mHelp_About});
			this.menu_Help.Text = "Help";
			// 
			// mHelp_About
			// 
			this.mHelp_About.Index = 0;
			this.mHelp_About.Text = "About";
			this.mHelp_About.Click += new System.EventHandler(this.mHelp_About_Click);
			// 
			// picPanel
			// 
			this.picPanel.BackColor = System.Drawing.Color.White;
			this.picPanel.Controls.AddRange(new System.Windows.Forms.Control[] {
																				   this.soup});
			this.picPanel.Name = "picPanel";
			this.picPanel.Size = new System.Drawing.Size(532, 532);
			this.picPanel.TabIndex = 1;
			// 
			// lbl_ip
			// 
			this.lbl_ip.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.lbl_ip.Location = new System.Drawing.Point(48, 24);
			this.lbl_ip.Name = "lbl_ip";
			this.lbl_ip.Size = new System.Drawing.Size(60, 23);
			this.lbl_ip.TabIndex = 2;
			// 
			// sb_mainWin
			// 
			this.sb_mainWin.Location = new System.Drawing.Point(0, 539);
			this.sb_mainWin.Name = "sb_mainWin";
			this.sb_mainWin.Panels.AddRange(new System.Windows.Forms.StatusBarPanel[] {
																						  this.sbp_Cycles,
																						  this.sbp_Ticks,
																						  this.sbp_Speed,
																						  this.sbp_Work,
																						  this.sbp_Load,
																						  this.sbp_NumSpecies,
																						  this.sbp_Threads,
																						  this.sbp_AvgSize});
			this.sb_mainWin.ShowPanels = true;
			this.sb_mainWin.Size = new System.Drawing.Size(792, 22);
			this.sb_mainWin.TabIndex = 3;
			// 
			// sbp_Cycles
			// 
			this.sbp_Cycles.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Spring;
			this.sbp_Cycles.BorderStyle = System.Windows.Forms.StatusBarPanelBorderStyle.Raised;
			this.sbp_Cycles.Text = "Cycles:";
			this.sbp_Cycles.ToolTipText = "The number of cycles that have elapsed.";
			this.sbp_Cycles.Width = 138;
			// 
			// sbp_Ticks
			// 
			this.sbp_Ticks.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Spring;
			this.sbp_Ticks.BorderStyle = System.Windows.Forms.StatusBarPanelBorderStyle.Raised;
			this.sbp_Ticks.Text = "Ticks:";
			this.sbp_Ticks.ToolTipText = "Number of ticks since last status update.";
			this.sbp_Ticks.Width = 138;
			// 
			// sbp_Speed
			// 
			this.sbp_Speed.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Spring;
			this.sbp_Speed.BorderStyle = System.Windows.Forms.StatusBarPanelBorderStyle.Raised;
			this.sbp_Speed.Text = "Speed:";
			this.sbp_Speed.ToolTipText = "The number of ticks executed per second.";
			this.sbp_Speed.Width = 138;
			// 
			// sbp_Work
			// 
			this.sbp_Work.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Spring;
			this.sbp_Work.BorderStyle = System.Windows.Forms.StatusBarPanelBorderStyle.Raised;
			this.sbp_Work.Text = "Work:";
			this.sbp_Work.ToolTipText = "The amount of work performed per second.";
			this.sbp_Work.Width = 138;
			// 
			// sbp_Load
			// 
			this.sbp_Load.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Contents;
			this.sbp_Load.Text = "Load:";
			this.sbp_Load.ToolTipText = "The percentage of the soup taken up by living cells.";
			this.sbp_Load.Width = 43;
			// 
			// sbp_NumSpecies
			// 
			this.sbp_NumSpecies.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Contents;
			this.sbp_NumSpecies.Text = "Species:";
			this.sbp_NumSpecies.ToolTipText = "The number of extant species.";
			this.sbp_NumSpecies.Width = 58;
			// 
			// sbp_Threads
			// 
			this.sbp_Threads.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Contents;
			this.sbp_Threads.Text = "Threads:";
			this.sbp_Threads.ToolTipText = "The number of active threads.";
			this.sbp_Threads.Width = 59;
			// 
			// sbp_AvgSize
			// 
			this.sbp_AvgSize.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Contents;
			this.sbp_AvgSize.Text = "Avg Size:";
			this.sbp_AvgSize.ToolTipText = "The average creature length.";
			this.sbp_AvgSize.Width = 62;
			// 
			// btn_Step
			// 
			this.btn_Step.Anchor = (System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right);
			this.btn_Step.BackColor = System.Drawing.Color.FromArgb(((System.Byte)(192)), ((System.Byte)(255)), ((System.Byte)(255)));
			this.btn_Step.Image = ((System.Drawing.Bitmap)(resources.GetObject("btn_Step.Image")));
			this.btn_Step.ImageAlign = System.Drawing.ContentAlignment.MiddleLeft;
			this.btn_Step.ImageIndex = 1;
			this.btn_Step.ImageList = this.imageList1;
			this.btn_Step.Location = new System.Drawing.Point(544, 464);
			this.btn_Step.Name = "btn_Step";
			this.btn_Step.Size = new System.Drawing.Size(72, 48);
			this.btn_Step.TabIndex = 4;
			this.btn_Step.Text = "Step";
			this.btn_Step.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
			this.btn_Step.Click += new System.EventHandler(this.btn_Step_Click);
			// 
			// imageList1
			// 
			this.imageList1.ColorDepth = System.Windows.Forms.ColorDepth.Depth32Bit;
			this.imageList1.ImageSize = new System.Drawing.Size(35, 35);
			this.imageList1.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageList1.ImageStream")));
			this.imageList1.TransparentColor = System.Drawing.Color.Transparent;
			// 
			// grp_registers
			// 
			this.grp_registers.Anchor = (System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right);
			this.grp_registers.BackColor = System.Drawing.Color.FromArgb(((System.Byte)(192)), ((System.Byte)(192)), ((System.Byte)(255)));
			this.grp_registers.Controls.AddRange(new System.Windows.Forms.Control[] {
																						this.label13,
																						this.lbl_fl,
																						this.label8,
																						this.lbl_inst,
																						this.label6,
																						this.label5,
																						this.lbl_sp,
																						this.lbl_dx,
																						this.lbl_cx,
																						this.lbl_bx,
																						this.lbl_ax,
																						this.lbl_ip,
																						this.label1,
																						this.label2,
																						this.label3,
																						this.label4});
			this.grp_registers.Location = new System.Drawing.Point(544, 168);
			this.grp_registers.Name = "grp_registers";
			this.grp_registers.Size = new System.Drawing.Size(248, 152);
			this.grp_registers.TabIndex = 5;
			this.grp_registers.TabStop = false;
			this.grp_registers.Text = "current state";
			// 
			// label13
			// 
			this.label13.BackColor = System.Drawing.Color.Transparent;
			this.label13.Location = new System.Drawing.Point(144, 120);
			this.label13.Name = "label13";
			this.label13.Size = new System.Drawing.Size(24, 23);
			this.label13.TabIndex = 14;
			this.label13.Text = "fl";
			// 
			// lbl_fl
			// 
			this.lbl_fl.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.lbl_fl.Location = new System.Drawing.Point(176, 120);
			this.lbl_fl.Name = "lbl_fl";
			this.lbl_fl.Size = new System.Drawing.Size(60, 23);
			this.lbl_fl.TabIndex = 13;
			// 
			// label8
			// 
			this.label8.BackColor = System.Drawing.Color.Transparent;
			this.label8.Location = new System.Drawing.Point(144, 24);
			this.label8.Name = "label8";
			this.label8.Size = new System.Drawing.Size(24, 23);
			this.label8.TabIndex = 11;
			this.label8.Text = "inst";
			// 
			// lbl_inst
			// 
			this.lbl_inst.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.lbl_inst.Location = new System.Drawing.Point(176, 24);
			this.lbl_inst.Name = "lbl_inst";
			this.lbl_inst.Size = new System.Drawing.Size(60, 23);
			this.lbl_inst.TabIndex = 10;
			// 
			// label6
			// 
			this.label6.BackColor = System.Drawing.Color.Transparent;
			this.label6.Location = new System.Drawing.Point(144, 88);
			this.label6.Name = "label6";
			this.label6.Size = new System.Drawing.Size(24, 23);
			this.label6.TabIndex = 9;
			this.label6.Text = "dx";
			// 
			// label5
			// 
			this.label5.BackColor = System.Drawing.Color.Transparent;
			this.label5.Location = new System.Drawing.Point(144, 53);
			this.label5.Name = "label5";
			this.label5.Size = new System.Drawing.Size(24, 23);
			this.label5.TabIndex = 8;
			this.label5.Text = "cx";
			// 
			// lbl_sp
			// 
			this.lbl_sp.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.lbl_sp.Location = new System.Drawing.Point(48, 120);
			this.lbl_sp.Name = "lbl_sp";
			this.lbl_sp.Size = new System.Drawing.Size(60, 23);
			this.lbl_sp.TabIndex = 7;
			// 
			// lbl_dx
			// 
			this.lbl_dx.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.lbl_dx.Location = new System.Drawing.Point(176, 88);
			this.lbl_dx.Name = "lbl_dx";
			this.lbl_dx.Size = new System.Drawing.Size(60, 23);
			this.lbl_dx.TabIndex = 6;
			// 
			// lbl_cx
			// 
			this.lbl_cx.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.lbl_cx.Location = new System.Drawing.Point(176, 56);
			this.lbl_cx.Name = "lbl_cx";
			this.lbl_cx.Size = new System.Drawing.Size(60, 23);
			this.lbl_cx.TabIndex = 5;
			// 
			// lbl_bx
			// 
			this.lbl_bx.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.lbl_bx.Location = new System.Drawing.Point(48, 88);
			this.lbl_bx.Name = "lbl_bx";
			this.lbl_bx.Size = new System.Drawing.Size(60, 23);
			this.lbl_bx.TabIndex = 4;
			// 
			// lbl_ax
			// 
			this.lbl_ax.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.lbl_ax.Location = new System.Drawing.Point(48, 56);
			this.lbl_ax.Name = "lbl_ax";
			this.lbl_ax.Size = new System.Drawing.Size(60, 23);
			this.lbl_ax.TabIndex = 3;
			// 
			// label1
			// 
			this.label1.BackColor = System.Drawing.Color.Transparent;
			this.label1.Location = new System.Drawing.Point(16, 24);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(24, 23);
			this.label1.TabIndex = 6;
			this.label1.Text = "ip";
			// 
			// label2
			// 
			this.label2.BackColor = System.Drawing.Color.Transparent;
			this.label2.Location = new System.Drawing.Point(16, 56);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(24, 23);
			this.label2.TabIndex = 7;
			this.label2.Text = "ax";
			// 
			// label3
			// 
			this.label3.BackColor = System.Drawing.Color.Transparent;
			this.label3.Location = new System.Drawing.Point(16, 88);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(24, 23);
			this.label3.TabIndex = 7;
			this.label3.Text = "bx";
			// 
			// label4
			// 
			this.label4.BackColor = System.Drawing.Color.Transparent;
			this.label4.Location = new System.Drawing.Point(16, 120);
			this.label4.Name = "label4";
			this.label4.Size = new System.Drawing.Size(24, 23);
			this.label4.TabIndex = 7;
			this.label4.Text = "sp";
			// 
			// groupBox1
			// 
			this.groupBox1.Anchor = (System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right);
			this.groupBox1.BackColor = System.Drawing.Color.FromArgb(((System.Byte)(192)), ((System.Byte)(192)), ((System.Byte)(255)));
			this.groupBox1.Controls.AddRange(new System.Windows.Forms.Control[] {
																					this.label12,
																					this.lbl_pfl,
																					this.label7,
																					this.lbl_pinst,
																					this.label10,
																					this.label11,
																					this.lbl_psp,
																					this.lbl_pdx,
																					this.lbl_pcx,
																					this.lbl_pbx,
																					this.lbl_pax,
																					this.lbl_pip,
																					this.label18,
																					this.label19,
																					this.label20,
																					this.label21});
			this.groupBox1.Location = new System.Drawing.Point(544, 8);
			this.groupBox1.Name = "groupBox1";
			this.groupBox1.Size = new System.Drawing.Size(248, 152);
			this.groupBox1.TabIndex = 6;
			this.groupBox1.TabStop = false;
			this.groupBox1.Text = "previous state";
			// 
			// label12
			// 
			this.label12.BackColor = System.Drawing.Color.Transparent;
			this.label12.Location = new System.Drawing.Point(144, 120);
			this.label12.Name = "label12";
			this.label12.Size = new System.Drawing.Size(24, 23);
			this.label12.TabIndex = 13;
			this.label12.Text = "fl";
			// 
			// lbl_pfl
			// 
			this.lbl_pfl.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.lbl_pfl.Location = new System.Drawing.Point(176, 120);
			this.lbl_pfl.Name = "lbl_pfl";
			this.lbl_pfl.Size = new System.Drawing.Size(60, 23);
			this.lbl_pfl.TabIndex = 12;
			// 
			// label7
			// 
			this.label7.BackColor = System.Drawing.Color.Transparent;
			this.label7.Location = new System.Drawing.Point(144, 24);
			this.label7.Name = "label7";
			this.label7.Size = new System.Drawing.Size(24, 23);
			this.label7.TabIndex = 11;
			this.label7.Text = "inst";
			// 
			// lbl_pinst
			// 
			this.lbl_pinst.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.lbl_pinst.Location = new System.Drawing.Point(176, 24);
			this.lbl_pinst.Name = "lbl_pinst";
			this.lbl_pinst.Size = new System.Drawing.Size(60, 23);
			this.lbl_pinst.TabIndex = 10;
			// 
			// label10
			// 
			this.label10.BackColor = System.Drawing.Color.Transparent;
			this.label10.Location = new System.Drawing.Point(144, 88);
			this.label10.Name = "label10";
			this.label10.Size = new System.Drawing.Size(24, 23);
			this.label10.TabIndex = 9;
			this.label10.Text = "dx";
			// 
			// label11
			// 
			this.label11.BackColor = System.Drawing.Color.Transparent;
			this.label11.Location = new System.Drawing.Point(144, 53);
			this.label11.Name = "label11";
			this.label11.Size = new System.Drawing.Size(24, 23);
			this.label11.TabIndex = 8;
			this.label11.Text = "cx";
			// 
			// lbl_psp
			// 
			this.lbl_psp.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.lbl_psp.Location = new System.Drawing.Point(48, 120);
			this.lbl_psp.Name = "lbl_psp";
			this.lbl_psp.Size = new System.Drawing.Size(60, 23);
			this.lbl_psp.TabIndex = 7;
			// 
			// lbl_pdx
			// 
			this.lbl_pdx.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.lbl_pdx.Location = new System.Drawing.Point(176, 88);
			this.lbl_pdx.Name = "lbl_pdx";
			this.lbl_pdx.Size = new System.Drawing.Size(60, 23);
			this.lbl_pdx.TabIndex = 6;
			// 
			// lbl_pcx
			// 
			this.lbl_pcx.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.lbl_pcx.Location = new System.Drawing.Point(176, 56);
			this.lbl_pcx.Name = "lbl_pcx";
			this.lbl_pcx.Size = new System.Drawing.Size(60, 23);
			this.lbl_pcx.TabIndex = 5;
			// 
			// lbl_pbx
			// 
			this.lbl_pbx.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.lbl_pbx.Location = new System.Drawing.Point(48, 88);
			this.lbl_pbx.Name = "lbl_pbx";
			this.lbl_pbx.Size = new System.Drawing.Size(60, 23);
			this.lbl_pbx.TabIndex = 4;
			// 
			// lbl_pax
			// 
			this.lbl_pax.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.lbl_pax.Location = new System.Drawing.Point(48, 56);
			this.lbl_pax.Name = "lbl_pax";
			this.lbl_pax.Size = new System.Drawing.Size(60, 23);
			this.lbl_pax.TabIndex = 3;
			// 
			// lbl_pip
			// 
			this.lbl_pip.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.lbl_pip.Location = new System.Drawing.Point(48, 24);
			this.lbl_pip.Name = "lbl_pip";
			this.lbl_pip.Size = new System.Drawing.Size(60, 23);
			this.lbl_pip.TabIndex = 2;
			// 
			// label18
			// 
			this.label18.BackColor = System.Drawing.Color.Transparent;
			this.label18.Location = new System.Drawing.Point(16, 24);
			this.label18.Name = "label18";
			this.label18.Size = new System.Drawing.Size(24, 23);
			this.label18.TabIndex = 6;
			this.label18.Text = "ip";
			// 
			// label19
			// 
			this.label19.BackColor = System.Drawing.Color.Transparent;
			this.label19.Location = new System.Drawing.Point(16, 56);
			this.label19.Name = "label19";
			this.label19.Size = new System.Drawing.Size(24, 23);
			this.label19.TabIndex = 7;
			this.label19.Text = "ax";
			// 
			// label20
			// 
			this.label20.BackColor = System.Drawing.Color.Transparent;
			this.label20.Location = new System.Drawing.Point(16, 88);
			this.label20.Name = "label20";
			this.label20.Size = new System.Drawing.Size(24, 23);
			this.label20.TabIndex = 7;
			this.label20.Text = "bx";
			// 
			// label21
			// 
			this.label21.BackColor = System.Drawing.Color.Transparent;
			this.label21.Location = new System.Drawing.Point(16, 120);
			this.label21.Name = "label21";
			this.label21.Size = new System.Drawing.Size(24, 23);
			this.label21.TabIndex = 7;
			this.label21.Text = "sp";
			// 
			// btn_Go
			// 
			this.btn_Go.Anchor = (System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right);
			this.btn_Go.BackColor = System.Drawing.Color.FromArgb(((System.Byte)(192)), ((System.Byte)(255)), ((System.Byte)(192)));
			this.btn_Go.Image = ((System.Drawing.Bitmap)(resources.GetObject("btn_Go.Image")));
			this.btn_Go.ImageAlign = System.Drawing.ContentAlignment.MiddleLeft;
			this.btn_Go.ImageIndex = 2;
			this.btn_Go.ImageList = this.imageList1;
			this.btn_Go.Location = new System.Drawing.Point(624, 464);
			this.btn_Go.Name = "btn_Go";
			this.btn_Go.Size = new System.Drawing.Size(72, 48);
			this.btn_Go.TabIndex = 7;
			this.btn_Go.Text = "Go";
			this.btn_Go.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
			this.btn_Go.Click += new System.EventHandler(this.btn_Go_Click);
			// 
			// btn_Dump
			// 
			this.btn_Dump.Anchor = (System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right);
			this.btn_Dump.BackColor = System.Drawing.Color.Tan;
			this.btn_Dump.Image = ((System.Drawing.Bitmap)(resources.GetObject("btn_Dump.Image")));
			this.btn_Dump.ImageAlign = System.Drawing.ContentAlignment.MiddleLeft;
			this.btn_Dump.Location = new System.Drawing.Point(704, 464);
			this.btn_Dump.Name = "btn_Dump";
			this.btn_Dump.Size = new System.Drawing.Size(72, 48);
			this.btn_Dump.TabIndex = 8;
			this.btn_Dump.Text = "Dump";
			this.btn_Dump.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
			this.btn_Dump.Click += new System.EventHandler(this.btn_Dump_Click);
			// 
			// dlg_SaveXML
			// 
			this.dlg_SaveXML.DefaultExt = "xml";
			this.dlg_SaveXML.FileName = "Soup";
			this.dlg_SaveXML.Filter = "XML Files (*.xml)|*.xml";
			this.dlg_SaveXML.InitialDirectory = ".";
			this.dlg_SaveXML.Title = "Save Soup to XML";
			// 
			// dlg_OpenXML
			// 
			this.dlg_OpenXML.DefaultExt = "xml";
			this.dlg_OpenXML.FileName = "Soup.xml";
			this.dlg_OpenXML.Filter = "XML Files (*.xml)|*.xml";
			this.dlg_OpenXML.InitialDirectory = ".";
			this.dlg_OpenXML.Title = "Restore XML to Soup";
			// 
			// mEdit_Configure
			// 
			this.mEdit_Configure.Index = 0;
			this.mEdit_Configure.Shortcut = System.Windows.Forms.Shortcut.CtrlG;
			this.mEdit_Configure.Text = "Configure";
			this.mEdit_Configure.Click += new System.EventHandler(this.mEdit_Configure_Click);
			// 
			// WinTierra
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(792, 561);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.btn_Dump,
																		  this.btn_Go,
																		  this.groupBox1,
																		  this.grp_registers,
																		  this.btn_Step,
																		  this.sb_mainWin,
																		  this.picPanel});
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.Menu = this.mainMenu1;
			this.MinimumSize = new System.Drawing.Size(688, 492);
			this.Name = "WinTierra";
			this.Text = "WinTierra";
			this.Resize += new System.EventHandler(this.WinTierra_Resize);
			this.Closing += new System.ComponentModel.CancelEventHandler(this.WinTierra_Closing);
			this.picPanel.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.sbp_Cycles)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.sbp_Ticks)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.sbp_Speed)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.sbp_Work)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.sbp_Load)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.sbp_NumSpecies)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.sbp_Threads)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.sbp_AvgSize)).EndInit();
			this.grp_registers.ResumeLayout(false);
			this.groupBox1.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		#region register Machine
		public void register( Machine newMa )
		{
			if( this.ma != null )
			{
				this.ma.MachineReport -= new MachineRefreshHandler(this.Machine_Report_Refresh_UI);
				this.ma.MachineRefresh -= new MachineRefreshHandler(this.Machine_Cycle_Refresh_UI);
				this.ma.MachineSingleStep -= new MachineStepHandler(this.Machine_Step_Refresh_UI);
				this.ma.MachineStop -= new MachineRefreshHandler(this.Machine_Stop_Refresh_UI);
				this.ma.mustStopCycling = true;
			}

			this.ma = null;
			bmpSoup = null;
			this.prevTicks = 0;
			this.prevWork = 0;
			this.myFileName = "";
			this.tre = null;

			if( newMa == null ) { return; }

			this.ma = newMa;
			newMa.MachineReport += new MachineRefreshHandler(this.Machine_Report_Refresh_UI);
			newMa.MachineRefresh += new MachineRefreshHandler(this.Machine_Cycle_Refresh_UI);
			newMa.MachineSingleStep += new MachineStepHandler(this.Machine_Step_Refresh_UI);
			newMa.MachineStop += new MachineRefreshHandler(this.Machine_Stop_Refresh_UI);

			if( censusf != null && ! censusf.isDisposed )
			{
				censusf.register(this.ma);
			}
			if( libf != null && ! libf.isDisposed )
			{
				libf.register(this.ma);
			}

			if( mView_AdultEmbryo.Checked )
			{
				ma.soupDisplayMode = 1;
			}
			else
			{
				ma.soupDisplayMode = 0;
			}

			newMa.onStop();
		}
		#endregion

		#region Machine Event Handlers
		private void Machine_Report_Refresh_UI(object sender, MachineState m)
		{
			statusBar_Refresh( m );
		}

		private void statusBar_Refresh( MachineState m)
		{
			if( ma == null ) { return; }
			int avgSize = 0;
			if( m.Population > 0 ) 
			{
				avgSize = m.Load / m.Population;
			}
			
			this.sbp_AvgSize.Text = "Avg Size: " + avgSize.ToString();
			this.sbp_Cycles.Text = "Cycles: " + m.Cycles.ToString("N", nfi);
			this.sbp_Ticks.Text = "Ticks: " + m.Ticks.ToString("N", nfi);
			this.sbp_Load.Text = "Load : " + ( ((double)m.Load / (double)m.Size) ).ToString("P", nfi);
			this.sbp_NumSpecies.Text = "# Species: " + m.NumSpecies.ToString("N", nfi);
			this.sbp_Threads.Text = "Threads: " + m.Population.ToString("N", nfi);
		}

		
		private void Machine_Cycle_Refresh_UI(object sender, MachineState m)
		{
			if( ma == null ) { return; }
			lock(soup)
			{
				statusBar_Refresh( m );
				ulong curTickCount = m.Ticks;
				ulong curWorkCount = m.Work;
				double durSince = myTimer.DurationSince;
				ulong ticksPerSec = (ulong)((curTickCount-prevTicks)/durSince);
				ulong workPerSec = (ulong)((curWorkCount-prevWork)/durSince);

//				debugLog += "Tick Diff: " + (curTickCount-prevTicks) + "\n";
//				debugLog += "Work Diff: " + (curWorkCount-prevWork) + "\n";
//				debugLog += "DurSince: " + durSince + "\n";
//				debugLog += "ticksPerSec: " + ticksPerSec + "\n";
//				debugLog += "workPerSec: " + workPerSec + "\n";
//				debugLog += "---------------------------\n";
				
				prevTicks = curTickCount;
				prevWork = curWorkCount;

				this.sbp_Speed.Text = "Speed: " + ticksPerSec.ToString("N", nfi) + " t/s";
				this.sbp_Work.Text = "Work: " + workPerSec.ToString("N", nfi) + " w/s";


				if( soup.Width == m.Width && soup.Height == m.Height )
				{
					if( bmpSoup != null && soup.Image == bmpSoup )
					{
						bmpSoup.Dispose();
						bmpSoup = null;
						if( gSoup != null )
						{
							gSoup.Dispose();
							gSoup = null;
						}
					}
					soup.Image = m.Snapshot;
				}
				else
				{
					if( soup.Width == 0 || soup.Height == 0 ) { return; }
					double xScaleFactor = (double)soup.Width / (double)m.Width;
					double yScaleFactor = (double)soup.Height / (double)m.Height;
					if( bmpSoup != null && ( soup.Width != bmpSoup.Width || soup.Height != bmpSoup.Height ) )
					{
						bmpSoup.Dispose();
						bmpSoup = null;
						if( gSoup != null )
						{
							gSoup.Dispose();
							gSoup = null;
						}
					}
					if( bmpSoup == null )
					{
						bmpSoup = new Bitmap(	(int)(m.Snapshot.Size.Width*xScaleFactor), (int)(m.Snapshot.Size.Height*yScaleFactor), m.Snapshot.PixelFormat);
						gSoup = Graphics.FromImage(bmpSoup);
						gSoup.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.NearestNeighbor;
						gSoup.PixelOffsetMode = System.Drawing.Drawing2D.PixelOffsetMode.HighSpeed;
						gSoup.ScaleTransform((float)xScaleFactor, (float)yScaleFactor);
						rectSoup = new Rectangle(0, 0, m.Snapshot.Size.Width, m.Snapshot.Size.Height);
					}
					gSoup.DrawImage(m.Snapshot, rectSoup, rectSoup, GraphicsUnit.Pixel);
					soup.Image = bmpSoup;
				}
			}		
		}


		private void Machine_Step_Refresh_UI(object sender, MachineThreadState e)
		{
			if( ma == null ) { return; }
			lbl_pax.Text = lbl_ax.Text;
			lbl_pbx.Text = lbl_bx.Text;
			lbl_pcx.Text = lbl_cx.Text;
			lbl_pdx.Text = lbl_dx.Text;
			lbl_pip.Text = lbl_ip.Text;
			lbl_psp.Text = lbl_sp.Text;
			lbl_pfl.Text = lbl_fl.Text;
			lbl_pinst.Text = lbl_inst.Text;

			lbl_ax.Text = e.Ergot.ax.ToString();
			lbl_bx.Text = e.Ergot.bx.ToString();
			lbl_cx.Text = e.Ergot.cx.ToString();
			lbl_dx.Text = e.Ergot.dx.ToString();
			lbl_ip.Text = e.Ergot.ip.ToString();
			lbl_sp.Text = e.Ergot.sp.ToString();
			lbl_fl.Text = e.Ergot.fl.ToString();
			lbl_inst.Text = e.Instruction;
			Machine_Cycle_Refresh_UI(sender, new MachineState(e.Ergot.myMachine));
		}


		private void Machine_Stop_Refresh_UI(object sender, MachineState m)
		{
			if( ma == null ) { return; }
			btn_Go.ImageIndex = 2;
			btn_Go.FlatStyle = System.Windows.Forms.FlatStyle.Standard;
			Machine_Cycle_Refresh_UI(sender, m);
			myTimer.Stop();
			//MessageBox.Show(this,debugLog,"Debug Log");
		}
		#endregion

		#region Button Event Handlers
		private void btn_Step_Click(object sender, System.EventArgs e)
		{
			if( ma.isCycling )
			{
				// stop thread
				ma.mustStopCycling = true;
				return;
			}
			if( ma.countAll == 0 )
			{
				// populate the soup if it's empty
				// so I stop getting false bug reports from dumbies who can't read directions :P
				geneLibRecord myGeneLibRecord = new geneLibRecord(Machine.c_default_creature, "" + ((int)Machine.c_default_creature.Length / 2) + "/ancestor");
				ma.addErgot( myGeneLibRecord.mySpeciesName, myGeneLibRecord.myAssembly, 0, myGeneLibRecord.mySize );
			}
			myTimer.Start();
			ma.runOneErgotOneTick();
			myTimer.Stop();
		}

		private void btn_Go_Click(object sender, System.EventArgs e)
		{
			if( ma.isCycling )
			{
				// stop thread
				ma.mustStopCycling = true;
				return;
			}
			if( ma.countAll == 0 )
			{
				// populate the soup if it's empty
				// so I stop getting false bug reports from dumbies who can't read directions :P
				geneLibRecord myGeneLibRecord = new geneLibRecord(Machine.c_default_creature, "" + ((int)Machine.c_default_creature.Length / 2) + "/ancestor");
				ma.addErgot( myGeneLibRecord.mySpeciesName, myGeneLibRecord.myAssembly, 35, myGeneLibRecord.mySize );
			}
			// start thread
			//debugLog += "Starting Machine...\n";
			this.btn_Go.ImageIndex = 0;
			this.btn_Go.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
			ma.mustStopCycling = false;

			if ((Control.ModifierKeys & Keys.Shift) == Keys.Shift)
			{
				//ma.wire = new System.Random(5555);
				tre = new Thread( new ThreadStart( ma.runErgotsOneHundredCycles ) );
			}
			else
			{
				tre = new Thread( new ThreadStart( ma.runErgots ) );
			}

			tre.Priority = System.Threading.ThreadPriority.Lowest;
			//debugLog = "";
			myTimer.Start();
			tre.Start();
		}

		private void btn_Dump_Click(object sender, System.EventArgs e)
		{
			if( ma.isCycling )
			{
				// stop thread
				ma.mustStopCycling = true;
				return;
			}
			if( this.myFileName == "" )
			{
				if(this.dlg_SaveXML.ShowDialog(this) != DialogResult.OK)
				{
					return;
				}
				myFileName = this.dlg_SaveXML.FileName;
				ma.ToXMLFile(myFileName);
				return;
			}
			string sLogDateTime	= DateTime.Now.ToLongTimeString();
			sLogDateTime = sLogDateTime.Replace( ':', '\'' );
			sLogDateTime = sLogDateTime.Replace( '/', ',' );
			sLogDateTime = sLogDateTime.Replace( ' ', '-' );
			string dumpFileName = myFileName;
			dumpFileName = System.Text.RegularExpressions.Regex.Replace( dumpFileName, "\\.([^\\.]*)\\Z", "-" + sLogDateTime + ".$1" );
			//MessageBox.Show(dumpFileName, "WinTierra", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
			ma.ToXMLFile(dumpFileName);
		}
		#endregion

		#region Window Event Handlers
		private void WinTierra_Resize(object sender, System.EventArgs e)
		{
			int picPanelHeight = this.Height - 64;
			if( picPanelHeight < 420 ) 
			{
				picPanelHeight = 420;
			}
			int picPanelWidth = picPanelHeight;
			if( this.Width - picPanelWidth < 264 )
			{
				picPanelWidth = this.Width - 264;
				picPanelHeight = picPanelWidth;
			}
			picPanel.Height = picPanelHeight;
			picPanel.Width = picPanelWidth;
			if( ! ma.isCycling )
			{
				Machine_Cycle_Refresh_UI( ma, new MachineState(ma) );
			}
			sb_mainWin.Text = "width: " + this.Width + ", height: " + this.Height;
		}

		private void WinTierra_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			if( ma.isCycling )
			{
				// stop thread
				ma.mustStopCycling = true;
			}
		}
		#endregion

		#region Drag & Drop Handlers
		private void soup_DragEnter(object sender, System.Windows.Forms.DragEventArgs e)
		{
			//See if there is a TreeNode being dragged
			if( e.Data.GetDataPresent("System.Windows.Forms.TreeNode", true) )
			{
				System.Windows.Forms.TreeNode dropNode =
					(System.Windows.Forms.TreeNode)
					e.Data.GetData(typeof(System.Windows.Forms.TreeNode));
				if( dropNode.ImageIndex == 2 )
				{
					// TreeNode of type 'GeneFile' found: allow move effect
					e.Effect = DragDropEffects.Move;
				}
				else
				{
					// TreeNode of type 'Folder' found: don't allow move effect.
					e.Effect = DragDropEffects.None;
				}
			}
			else
			{
				// No TreeNode found, prevent move
				e.Effect = DragDropEffects.None;
			}
		}

		private void soup_DragDrop(object sender, System.Windows.Forms.DragEventArgs e)
		{
			//See if there is a TreeNode being dropped
			if( e.Data.GetDataPresent("System.Windows.Forms.TreeNode", true) )
			{
				System.Windows.Forms.TreeNode dropNode =
					(System.Windows.Forms.TreeNode)
					e.Data.GetData(typeof(System.Windows.Forms.TreeNode));
				if( dropNode.ImageIndex == 2 )
				{
					// TreeNode of type 'GeneFile' found: insert its tag into the soup.
					geneLibRecord myGeneLibRecord = (geneLibRecord)dropNode.Tag;
					if( myGeneLibRecord != null )
					{
						Point clientPoint = soup.PointToClient(new Point(e.X, e.Y));
						double xScaleFactor = (double)soup.Width / (double)ma.width;
						double yScaleFactor = (double)soup.Height / (double)ma.height;
						
						int xActual = (int)((double)clientPoint.X / (double)xScaleFactor);
						int yActual = (int)((double)clientPoint.Y / (double)yScaleFactor);
						
						int myDropPosition = (int)((yActual * ma.width ) + xActual);
//						MessageBox.Show("Inserting " + myGeneLibRecord.myGenome + " at position: "
//							+ "(clientX,clientY): " + clientPoint.X + ", " + clientPoint.Y
//							+ ", (xScaleFactor,yScaleFactor): " + xScaleFactor + ", " + yScaleFactor
//							+ ", (xActual,yActual): " + xActual + ", " + yActual
//							+ " = " + myDropPosition + "."
//							, "WinTierra", MessageBoxButtons.OK, MessageBoxIcon.Error);
						if( myDropPosition >= 0 && myDropPosition < ma.size )
						{
							ma.addErgot( myGeneLibRecord.mySpeciesName, myGeneLibRecord.myAssembly, myDropPosition, myGeneLibRecord.mySize );
						}
					}
				}
				else
				{
					// TreeNode of type 'Folder' found: ignore.
				}
			}
			else
			{
				// No TreeNode found, prevent move
				e.Effect = DragDropEffects.None;
			}
		}
		#endregion

		#region File Menu Item Event Handlers
		private void mFile_New_Click(object sender, System.EventArgs e)
		{
			int defaultSoupSizeRoot = 0;
			bool allowCopyConfig = false;
			if( ma != null && ma.isValid )
			{
				defaultSoupSizeRoot = ma.opSoupSizeRoot;
				allowCopyConfig = true;
			}
			NewSoup ns = new NewSoup(defaultSoupSizeRoot, allowCopyConfig);
			ns.ShowDialog(this);
			if( ns.DialogResult == System.Windows.Forms.DialogResult.OK )
			{
				int newSoupSize = ns.newSoupSize;
				Machine newMa = new Machine();
				if( ma != null && ma.isValid && ns.copyConfig )
				{
					MachineOptions mao = new MachineOptions( ma );
					mao.ApplyOptions(newMa);
				}
				newMa.opSoupSizeRoot = newSoupSize;
				newMa.initialize();
				if( newMa.isValid )
				{
					this.register( newMa );
				}
				else
				{
					MessageBox.Show(this,newMa.ErrorString,"Machine Initialization Error",System.Windows.Forms.MessageBoxButtons.OK,System.Windows.Forms.MessageBoxIcon.Error);
				}
				GC.Collect();
			}
		}

		private void mFile_Save_Click(object sender, System.EventArgs e)
		{
			if( ma.isCycling )
			{
				// stop thread
				ma.mustStopCycling = true;
				return;
			}
			if( this.myFileName == "" )
			{
				if(this.dlg_SaveXML.ShowDialog(this) != DialogResult.OK)
				{
					return;
				}
				myFileName = this.dlg_SaveXML.FileName;
			}
			ma.ToXMLFile(myFileName);
			
			if( ma.ErrorCode != 0 )
			{
				// display error.
				MessageBox.Show(ma.ErrorString, "WinTierra", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void mFile_SaveAs_Click(object sender, System.EventArgs e)
		{
			if( ma.isCycling )
			{
				// stop thread
				ma.mustStopCycling = true;
				return;
			}
			if(this.dlg_SaveXML.ShowDialog(this) != DialogResult.OK)
			{
				return;
			}
			myFileName = this.dlg_SaveXML.FileName;
			ma.ToXMLFile(myFileName);

			if( ma.ErrorCode != 0 )
			{
				// display error.
				MessageBox.Show(ma.ErrorString, "WinTierra", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void mFile_Open_Click(object sender, System.EventArgs e)
		{
			if( ma.isCycling )
			{
				// stop thread
				ma.mustStopCycling = true;
				return;
			}
			if(this.dlg_OpenXML.ShowDialog(this) != DialogResult.OK)
			{
				return;
			}
			myFileName = this.dlg_OpenXML.FileName;
			Machine newMa = new Machine(myFileName);
			if( newMa.isValid )
			{
				this.register(newMa);
			}
			else
			{
				// display error.
				MessageBox.Show(newMa.ErrorString, "WinTierra", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void mFile_Exit_Click(object sender, System.EventArgs e)
		{
			this.Close();
		}
		#endregion

		#region View Menu Item Event Handlers
		private void mView_AdultEmbryo_Click(object sender, System.EventArgs e)
		{
			mView_Normal.Checked = mView_AdultEmbryo.Checked;
			mView_AdultEmbryo.Checked = ! mView_AdultEmbryo.Checked;
			if( mView_AdultEmbryo.Checked )
			{
				ma.soupDisplayMode = 1;
			}
			else
			{
				ma.soupDisplayMode = 0;
			}
		}

		private void mView_Normal_Click(object sender, System.EventArgs e)
		{
			mView_Normal.Checked = mView_AdultEmbryo.Checked;
			mView_AdultEmbryo.Checked = ! mView_AdultEmbryo.Checked;
			if( mView_Normal.Checked )
			{
				ma.soupDisplayMode = 0;
			}
			else
			{
				ma.soupDisplayMode = 1;
			}
		}

		private void mView_GeneLibrary_Click(object sender, System.EventArgs e)
		{
			if( libf == null || libf.DialogResult == DialogResult.Abort )
			{
				libf = new LibraryForm();
				libf.register(ma);
				libf.Show();
			}
			else
			{
				libf.Activate();
			}
		}

		private void mView_Census_Click(object sender, System.EventArgs e)
		{
			if( censusf == null || censusf.DialogResult == DialogResult.Abort )
			{
				censusf = new CensusForm();
				censusf.register(ma);
				censusf.Show();
			}
			else
			{
				censusf.Activate();
			}
		}
		#endregion

		#region Help Menu Item Event Handlers
		private void mHelp_About_Click(object sender, System.EventArgs e)
		{
			string aboutstr = "ERGOT: MacTierra for Windows";
			aboutstr += "\ncopyright 2006 by plushpuffin@gmail.com";
			aboutstr += "\nhttp://www.wwddfd.com/";
			aboutstr += "\n\nlatest build: http://www.wwddfd.com/frell/ergot.zip";
			aboutstr += "\n\nlicensing terms still under consideration...";
			MessageBox.Show(aboutstr, "WinTierra", MessageBoxButtons.OK, MessageBoxIcon.Information);
		}
		#endregion

		#region Edit Menu Item Event Handlers
		private void mEdit_Configure_Click(object sender, System.EventArgs e)
		{
			Configure con = new Configure();
			con.register( ma );
			con.ShowDialog(this);
		}
		#endregion

	}
}
