//
// ERGOT : The Windows MacTierra clone
// copyright (c) 2005 plushpuffin@gmail.com
// http://www.wwddfd.com/
//
// latest build: http://www.wwddfd.com/frell/ergot.zip
//
// licensing terms still under consideration...

using System;

namespace KTL
{
	[Serializable]
	public class Node
	{
		#region Private Member Variables
		private object	myValue;
		private Node	nextNode;
		private Node	previousNode;
		#endregion

		#region Constructors and Destructor
		public Node(object newValue)
		{
			this.myValue = newValue;
		}

		public Node(object newValue, Node nextNode, Node previousNode)
		{
			this.myValue		= newValue;
			this.nextNode		= nextNode;
			this.previousNode	= previousNode;
		}
		#endregion

		//		~Node()
		//		{
		//			System.Windows.Forms.MessageBox.Show("A KTL.Node is being destroyed!", "KTL.Node", System.Windows.Forms.MessageBoxButtons.OK, System.Windows.Forms.MessageBoxIcon.Error);
		//		}

		#region Public Member Properties
		public object Value
		{
			get
			{
				return myValue;
			}

			set
			{
				myValue = value;
			}
		}

		public Node NextNode
		{
			get
			{
				return nextNode;
			}

			set
			{
				nextNode = value;
			}
		}

		public Node PrevNode
		{
			get
			{
				return previousNode;
			}

			set
			{
				previousNode = value;
			}
		}

		public bool isNull
		{
			get
			{
				return myValue == null;
			}
		}
		#endregion
	}

	/// <summary>
	/// Summary description for ReaperQueue.
	/// </summary>
	public class dList
	{

		#region Private Member Variables
		private Node headerNode;
		private int count;
		#endregion

		#region Public Constructors
		public dList()
		{
			headerNode = new Node(null, null, null);
			headerNode.NextNode	= headerNode;
			headerNode.PrevNode	= headerNode;
			count = 0;
		}
		#endregion

		#region Public Member Properties
		public Node Head
		{
			get
			{
				return headerNode.NextNode;
			}
		}

		public Node Tail
		{
			get
			{
				return headerNode.PrevNode;
			}
		}

		public bool isEmpty
		{
			get
			{
				return headerNode.NextNode == headerNode;
			}
		}

		public int Count
		{
			get
			{
				return count;
			}
		}
		#endregion

		#region Queue operations
		public Node shift()
		{
			if( headerNode.NextNode == headerNode ) { return null; }
			return removeNode(headerNode.NextNode);
		}

		public void unshift( Node newNode )
		{
			if( newNode == null ) { return; }
			insertNodeAfter( newNode, headerNode );
		}
		#endregion

		#region Stack operations
		public Node pop()
		{
			if( headerNode.NextNode == headerNode ) { return null; }
			return removeNode(headerNode.PrevNode);
		}

		public void push( Node newNode )
		{
			if( newNode == null ) { return; }
			insertNodeAfter( newNode, headerNode.PrevNode );
		}
		#endregion

		#region Reordering
		public void moveUp( Node thisNode )
		{
			if( thisNode == null || thisNode.PrevNode == headerNode ) { return; }
			removeNode( thisNode );
			insertNodeAfter( thisNode, thisNode.PrevNode.PrevNode );
		}

		public void moveDown( Node thisNode )
		{
			if( thisNode == null || thisNode.NextNode == headerNode ) { return; }
			removeNode( thisNode );
			insertNodeAfter( thisNode, thisNode.NextNode );
		}
		#endregion

		#region Insertion and Removal
		public void insertNodeAfter( Node newNode, Node thisNode )
		{
			if( thisNode == null || newNode == headerNode ) { return; }
			newNode.NextNode = thisNode.NextNode;
			newNode.PrevNode = thisNode;
			thisNode.NextNode.PrevNode = newNode;
			thisNode.NextNode = newNode;
			++count;
		}

		public Node removeNode( Node thisNode )
		{
			if( thisNode == null || thisNode == headerNode ) { return null; }
			thisNode.PrevNode.NextNode = thisNode.NextNode;
			thisNode.NextNode.PrevNode = thisNode.PrevNode;
			--count;
			return thisNode;
		}
		#endregion

	}
}
